<?php
/**
 * @file
 * Views integration for Conditional roles by taxonomy.
 */

/**
 * Implements hook_views_data().
 */
function conditional_roles_taxonomy_views_data() {
  $data = array();

  // Find any conditions provided by this module that are currently enforced.
  $info = conditional_roles_taxonomy_conditional_roles_info();
  $enforced_conditions = variable_get('conditional_roles_enforced_conditions', array());
  foreach (array_keys($enforced_conditions) as $cid) {
    if (isset($info[$cid])) {
      if ($cid == 'taxonomy_term') {
        // @todo Uncomment once function is implemented.
        //$data += _conditional_roles_taxonomy_build_term_views_data();
      }
      else {
        $field = field_info_field($cid);
        if (!empty($field) && $field['storage']['type'] == 'field_sql_storage') {
          $data += _conditional_roles_taxonomy_build_field_views_data($field);
        }
      }
    }
  }

  return $data;
}

/**
 * Helper function to build views data for taxonomy term conditions.
 */
function _conditional_roles_taxonomy_build_term_views_data() {
  // @todo Add support for taxonomy_term conditions.
}

/**
 * Helper function to build views data for term reference field conditions.
 *
 * @param $field
 *   A taxonomy_term_reference field.
 */
function _conditional_roles_taxonomy_build_field_views_data($field) {
  $data = array();

  // Get the current and revision tables.
  $tables = array();
  $tables[FIELD_LOAD_CURRENT] = _field_sql_storage_tablename($field);
  $tables[FIELD_LOAD_REVISION] = _field_sql_storage_revision_tablename($field);

  foreach ($tables as $type => $table) {
    // Get the name of the table column that contains the field value.
    $column = $field['storage']['details']['sql'][$type][$table]['tid'];

    // Declare a virtual field on the table for testing role application.
    $data[$table]['conditional_roles_taxonomy_role'] = array(
      'group' => t('Conditional roles'),
      'title' => t('Current user has role based on @field', array('@field' => $field['field_name'])),
      'help' => t('Tests whether the values in @field permit the current user to exercise the permissions granted by a selected role.', array('@field' => $field['field_name'])),
      'real field' => $column,
      'filter' => array(
        'handler' => 'conditional_roles_taxonomy_handler_filter_role',
        'field_name' => $field['field_name'],
      ),
    );

    // Declare a virtual field on the table for testing permission application.
    $data[$table]['conditional_roles_taxonomy_permission'] = array(
      'group' => t('Conditional roles'),
      'title' => t('Current user has permission based on @field', array('@field' => $field['field_name'])),
      'help' => t('Tests whether the values in @field permit the current user to exercise the selected permission.', array('@field' => $field['field_name'])),
      'real field' => $column,
      'filter' => array(
        'handler' => 'conditional_roles_taxonomy_handler_filter_permission',
        'field_name' => $field['field_name'],
      ),
    );
  }

  return $data;
}
