<?php
/**
 * @file
 * Provides the Conditional roles permission filter for Views.
 */

/**
 * Filter by whether or not the current user has a permission on a node.
 */
class conditional_roles_taxonomy_handler_filter_permission extends views_handler_filter_many_to_one {
  function get_value_options() {
    $permissions = module_invoke_all('permission');
    $this->value_options = array();
    foreach ($permissions as $perm => $info) {
      $this->value_options[$perm] = check_plain(strip_tags($info['title']));
    }
  }

  function options_form(&$form, &$form_state) {
    parent::options_form($form, $form_state);
    // This filter can cause duplicates, but is not currently compatible with
    // the 'Reduce duplicates' option.
    $form['reduce_duplicates']['#title'] .= ' ' . t('(not supported)');
    $form['reduce_duplicates']['#disabled'] = TRUE;
  }

  function operators() {
    $operators = parent::operators();
    // @todo The 'and' operator is complicated and hasn't been written yet.
    unset($operators['and']);
    // @todo The 'not' operator isn't working for some reason.
    unset($operators['not']);
    return $operators;
  }

  function query() {
    global $user;
    $this->ensure_my_table();
    $cid = $this->definition['field_name'];
    $perms = is_array($this->value) ? $this->value : array_keys($this->value_options);

    // Get the conditions on each selected permission.
    $perm_conditions = array();
    foreach ($perms as $perm) {
      $perm_conditions[$perm] = _conditional_roles_user_permissions($perm, $user);
    }

    $operator = ($this->operator == 'not') ? 'NOT IN' : 'IN';
    switch ($this->operator) {
      case 'and':
        // @todo This needs to handle both additive cases (two roles based on
        //   two different terms both applying to a node) and subtractive cases
        //   (one roles is based on two terms, but the second role is only based
        //   on one, so only show nodes that have the shared term).
        break;

      case 'or':
      case 'not':
        $tids = array();
        foreach ($perm_conditions as $conditions) {
          if ($conditions === TRUE) {
            // The user holds the permission unconditionally, so there's no need to
            // keep building a list of tids, since it applies to all nodes.
            $tids = TRUE;
            break;
          }
          elseif (!is_array($conditions) || empty($conditions[$cid])) {
            // The user doesn't hold the permission.
            continue;
          }

          $tids += $conditions[$cid];
        }

        if ($tids === TRUE || empty($tids)) {
          // If all or no tids match, do a straight comparison based on the op.
          $have = empty($tids) ? 0 : 1;
          $need = ($this->operator == 'not') ? 0 : 1;
          $this->query->add_where($this->options['group'], $have, $need);
        }
        else {
          $this->query->add_where($this->options['group'], "$this->table_alias.$this->real_field", $tids, $operator);
          // @todo Find out why this works with IN but fails with NOT IN...
          //   Is it related to the :views_join_condition_0 clause?
        }
        break;
    }
  }
}
